#!/bin/ksh

#**************************************************************
# Author: Sebastien Denvil, Martial Mancip
# Contact: Sebastien.Denvil__at__ipsl.jussieu.fr
# $Revision::                                          $ Revision of last commit
# $Author::                                            $ Author of last commit
# $Date::                                              $ Date of last commit
# IPSL (2006)
#  This software is governed by the CeCILL licence see libIGCM/libIGCM_CeCILL.LIC
#
#**************************************************************

#=========================================================
# The documentation of this file can be automatically generated
# if you use the prefix #D- for comments to be extracted. 
# Extract with command: cat lib* | grep "^#D-" | cut -c "4-"
#=========================================================

#D-#==================================================
#D-LibIGCM_sys for Mercure SX9
#D-#==================================================
#D-
#D- This ksh library if a layer under some usefull 
#D-environment variables and shell commands.
#D-All those definitions depend on host particularities.
#D-It manages a stack mechanism and test validity of operations.
#D-All function described bellow must be prefixed by IGCM_sys.

#====================================================
# libIGCM_sys PARAMETERS
#====================================================

# gawk specific location on SX9
export PATH=/applications/gawk-3.0.4/bin:${PATH}

#====================================================
# set DEBUG_sys to true to output calls of function
typeset -r DEBUG_sys=${DEBUG_sys:=true}

#====================================================
# Turn in dry run mode ? (sys_Put_Rest, sys_Put_Out, sys_Get)
typeset -r DRYRUN=${DRYRUN:=0}

# YOU MUST COMPILE YOUR EXE FILES FOR DRYRUN MODE !
# -------------------------------------------------------------------------------------
# | DRYRUN=  |  Date computations, | sys_Get    |  Exe    | sys_Put_Out; sys_Put_Rest |
# |          |  Cp/Exe param files |            |  Chmod  |                           |
# |          |      Qsub           |            |         |                           |
# -------------------------------------------------------------------------------------
# |    0     |       yes           |    yes     |  yes    |      yes                  |
# -------------------------------------------------------------------------------------
# |    1     |       yes           |    yes     |  yes    |      no                   |
# -------------------------------------------------------------------------------------
# |    2     |       yes           |    yes     |  no     |      no                   |
# -------------------------------------------------------------------------------------
# |    3     |       yes           |    no      |  no     |      no                   |
# -------------------------------------------------------------------------------------

#=====================================================
# Global Variables :
#=====================================================
# Language : "fr" or "en"
typeset -r MYLANG="fr"

#=====================================================
# Host and user names
# $hostname ou hostname
typeset  HOST=${HOST:=$( hostname )}
# $username ou whoami
typeset  LOGIN=${LOGIN:=$( whoami )}
# $hostname of the MASTER job
typeset -r MASTER=mercure

#D-
#D-#==================================================
#D-Program used in libIGCM
#D-#==================================================

# rsync with path
typeset -r RSYNC=/home/cont003/p86denv/SX_RSYNC/bin/rsync
# RSYNC_opt args to rsync
typeset -r RSYNC_opt="-va"
# ie storage filesystem
typeset -r STOREHOST=${MASTER}

#====================================================
# Host specific DIRECTORIES
#====================================================

# ============ CESIUM START ============ #

#====================================================
#- Mirror libIGCM from mercure to cesium if needed
#ROOTSYS=$( echo ${libIGCM} | gawk -F"/" '{print $3}' )
#if [ ! ${ROOTSYS} = "home" ] ; then
#  typeset -r MirrorlibIGCM=${MirrorlibIGCM:=true}
#else
#  typeset -r MirrorlibIGCM=${MirrorlibIGCM:=false}
#fi

#====================================================
#- libIGCM_POST
#if ( ${MirrorlibIGCM} ) ; then
#  PATHlibIGCM=$( echo ${libIGCM} | gawk -F"${LOGIN}/" '{print $2}' | sed -e "s&/libIGCM&&" )
#  typeset -r libIGCM_POST=${HOME}/MIRROR/${PATHlibIGCM}/libIGCM
#else
#  typeset -r libIGCM_POST=${libIGCM}
#fi

# ============ CESIUM  END  ============ #

#====================================================
#- MirrorlibIGCM uncomment for frontend
typeset -r MirrorlibIGCM=${MirrorlibIGCM:=false}

#====================================================
#- libIGCM_POST uncomment for frontend
typeset -r libIGCM_POST=${libIGCM}

#====================================================
#- R_EXE   (==> BIN_DIR = ${MODIPSL}/bin )
typeset -r R_EXE="${MODIPSL}/bin"

#====================================================
#- SUBMIT_DIR : submission dir
typeset SUBMIT_DIR=${SUBMIT_DIR:=${PBS_O_WORKDIR}}

#====================================================
#- ARCHIVE (dedicated to large files)
typeset -r ARCHIVE=${CCCSTOREDIR}

#- ARCHIVE (dedicated to small/medium files)
typeset -r STORAGE=${CCCWORKDIR}

#====================================================
#- IN
typeset -r R_IN=${R_IN:=/ccc/work/cont003/dsm/p86ipsl/IGCM}
typeset -r R_IN_ECMWF=${R_IN_ECMWF:=/dmnfs/cont003/p24data}

#====================================================
#- R_OUT
typeset -r R_OUT=${ARCHIVE}/IGCM_OUT

#====================================================
#- R_FIG (hosting figures : monitoring and atlas, and/or small files)
typeset -r R_FIG=${STORAGE}/IGCM_OUT

#====================================================
#- R_BUF (ONLY FOR double copy an scratch)
typeset -r R_BUF=${SCRATCHDIR}/IGCM_OUT

#====================================================
#- BIG_DIR : BIG_DIR to store files waiting for rebuild
typeset -r BIG_DIR=${BIG_DIR:=${SCRATCHDIR}/REBUILD}

#====================================================
#- OUT_POST
typeset -r R_OUT_POST=${SCRATCHDIR}/IGCM_OUT

#====================================================
#- RUN_DIR_PATH : Temporary working directory (=> TMP)
typeset -r RUN_DIR_PATH=${RUN_DIR_PATH:=${LOCALTMPDIR}}

#====================================================
#- HOST_MPIRUN_COMMAND
typeset -r HOST_MPIRUN_COMMAND=${HOST_MPIRUN_COMMAND:="mpirun"}

#====================================================
#- Max number of arguments passed to nco operator or demigration command
UNIX_MAX_LIMIT=360

#====================================================
#- set PackDefault true on NEC SX9
PackDefault=true


#====================================================
#- Default number of MPI task for IPSL coupled model
#- required for backward compatibility
#-
DEFAULT_NUM_PROC_OCE=1
DEFAULT_NUM_PROC_CPL=1
(( DEFAULT_NUM_PROC_ATM = BATCH_NUM_PROC_TOT - 1 ))
DEFAULT_NUM_PROC_TOTAL=${BATCH_NUM_PROC_TOT}

#D-#==================================================
#D-function IGCM_sys_RshMaster
#D-* Purpose: Connection to frontend machine.
#D-* Examples:
#D-
function IGCM_sys_RshMaster {
  IGCM_debug_PushStack "IGCM_sys_RshMaster" $@
  ssh -t ${MASTER} /bin/ksh <<-EOF
  export libIGCM=${libIGCM}
  export DEBUG_debug=${DEBUG_debug}
  . ${libIGCM}/libIGCM_debug/libIGCM_debug.ksh
  . ${libIGCM}/libIGCM_card/libIGCM_card.ksh
  ${@}
EOF
  if [ $? -gt 0 ] ; then
    echo "IGCM_sys_RshMaster : erreur."
    IGCM_debug_Exit "IGCM_sys_RshMaster"
  fi
  IGCM_debug_PopStack "IGCM_sys_RshMaster"
}

#D-#==================================================
#D-function IGCM_sys_RshArchive
#D-* Purpose: Archive rsh command
#D-* Examples:
#D-
function IGCM_sys_RshArchive {
  IGCM_debug_PushStack "IGCM_sys_RshArchive" $@
  /bin/ksh <<-EOF
    ${@}
EOF
  if [ $? -gt 0 ] ; then
    echo "IGCM_sys_RshArchive : erreur."
    IGCM_debug_Exit "IGCM_sys_RshArchive"
  fi
  IGCM_debug_PopStack "IGCM_sys_RshArchive"
}

#D-#==================================================
#D-function IGCM_sys_RshPost
#D-* Purpose: Post-process rsh command
#D-* Examples:
#D-
function IGCM_sys_RshPost {
  IGCM_debug_PushStack "IGCM_sys_RshPost" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_RshPost :" $@
  fi

  #echo cat tmp_IGCM_sys_RshPost_$$ INITIAL
  #cat tmp_IGCM_sys_RshPost_$$
  # keep standard input (stdin) for the loop onto temporary file
  cat >tmp_IGCM_sys_RshPost_$$

# ============ FRONTEND START ============ #

  /bin/ksh <tmp_IGCM_sys_RshPost_$$
  if [ $? -gt 0 ] ; then
    echo "IGCM_sys_RshPost : erreur."
    IGCM_debug_Exit "IGCM_sys_RshPost"
  fi
  \rm tmp_IGCM_sys_RshPost_$$

# ============ FRONTEND  END  ============ #

# ============ CESIUM START ============ #
#  typeset NB_ESSAI DELAI status i
#  # number of tentative
#  NB_ESSAI=10
#  # time delay between tentative
#  DELAI=10
#  (( i = 0 ))
#  while [ $i -lt $NB_ESSAI ] ; do
#    ssh -t mercure01 ssh cesium /bin/ksh <tmp_IGCM_sys_RshPost_$$
#    status=$?
#    if [ ${status} -gt 0 ]; then
#      IGCM_debug_Print 2 "IGCM_sys_RshPost : ssh failed ${i}/${NB_ESSAI}"
#      IGCM_debug_Print 2 "IGCM_sys_RshPost : sleep ${DELAI} seconds and try again."
#      sleep $DELAI
#    else
#      break
#    fi
#    (( i = i + 1 ))
#  done 
#  # delete temporary file
#  /bin/rm tmp_IGCM_sys_RshPost_$$
# ============ CESIUM  END  ============ #

  IGCM_debug_PopStack "IGCM_sys_RshPost"
}

#D-#==================================================
#D-function IGCM_sys_SendMail
#D-* Purpose: Send mail when simulation is over
#D-* Examples:
#D-
function IGCM_sys_SendMail {
  IGCM_debug_PushStack "IGCM_sys_SendMail" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_SendMail :" $@
  fi

  if ( ${ExitFlag} ) ; then
    status=failed
  else
    status=completed
  fi
  cat  << END_MAIL > job_end.mail
  Dear ${LOGIN},

  Simulation ${config_UserChoices_JobName} is ${status} on supercomputer `hostname`.
  Job started : ${DateBegin}
  Job ended   : ${DateEnd}
  Output files are available in ${R_SAVE}
  Pre-packed files are available in ${R_BUFR}
  Script files, Script Outputs and Debug files (if necessary) are available in ${SUBMIT_DIR}
END_MAIL

  if  [ X"${config_UserChoices_MailName}" != X ] ; then
    mailx -s "${config_UserChoices_JobName} ${status}" ${config_UserChoices_MailName} <  job_end.mail
  elif [ -f ~/.forward ] ; then
    mailx -s "${config_UserChoices_JobName} ${status}" $( cat ~/.forward ) < job_end.mail
  else
    mailx -s "${config_UserChoices_JobName} ${status}" ${LOGIN} < job_end.mail
  fi

  if [ $? -gt 0 ] ; then
    echo "IGCM_sys_SendMail : erreur."
    IGCM_debug_Exit "IGCM_sys_SendMail"
  fi
  IGCM_debug_PopStack "IGCM_sys_SendMail"
}

#D-#==================================================
#D-function IGCM_sys_Mkdir
#D-* Purpose: Master locale mkdir command
#D-* Examples:
#D-
function IGCM_sys_Mkdir {
  IGCM_debug_PushStack "IGCM_sys_Mkdir" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Mkdir :" $@
  fi
  if [ ! -d ${1} ]; then
    \mkdir -p $1
    if [ $? -gt 0 ] ; then
      echo "IGCM_sys_Mkdir : erreur."
      IGCM_debug_Exit "IGCM_sys_Mkdir"
    fi
  fi
  # vrification :
  if [ ! -d ${1} ] ; then
    echo "IGCM_sys_Mkdir : erreur."
    IGCM_debug_Exit "IGCM_sys_Mkdir"
  fi
  IGCM_debug_PopStack "IGCM_sys_Mkdir"
}

#D-#==================================================
#D-function IGCM_sys_MkdirArchive
#D-* Purpose: Mkdir on Archive
#D-* Examples:
#D-
function IGCM_sys_MkdirArchive {
  IGCM_debug_PushStack "IGCM_sys_MkdirArchive" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_MkdirArchive :" $@
  fi
  #- creation de repertoire sur le serveur fichier
  if [ ! -d ${1} ]; then 
    \mkdir -p $1
    if [ $? -gt 0 ] ; then
      echo "IGCM_sys_MkdirArchive : erreur."
      IGCM_debug_Exit "IGCM_sys_MkdirArchive"
    fi
  fi
  IGCM_debug_PopStack "IGCM_sys_MkdirArchive"
}

#D-#==================================================
#D-function IGCM_sys_MkdirWork
#D-* Purpose: Mkdir on Work
#D-* Examples:
#D-
function IGCM_sys_MkdirWork {
  IGCM_debug_PushStack "IGCM_sys_MkdirWork" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_MkdirWork :" $@
  fi
  #- creation de repertoire sur le serveur fichier
  if [ ! -d ${1} ]; then 
    \mkdir -p $1
    if [ $? -gt 0 ] ; then
      echo "IGCM_sys_MkdirWork : erreur."
      IGCM_debug_Exit "IGCM_sys_MkdirWork"
    fi
  fi
  IGCM_debug_PopStack "IGCM_sys_MkdirWork"
}

#D-#==================================================
#D-function IGCM_sys_Cd
#D-* Purpose: master cd command
#D-* Examples:
#D-
function IGCM_sys_Cd {
  IGCM_debug_PushStack "IGCM_sys_Cd" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Cd :" $@
  fi
  \cd $1
  if [ $? -gt 0 ] ; then
    echo "IGCM_sys_Cd : erreur."
    IGCM_debug_Exit "IGCM_sys_Cd"
  fi
  IGCM_debug_PopStack "IGCM_sys_Cd"
}

#D-#==================================================
#D-function IGCM_sys_Chmod
#D-* Purpose: Chmod
#D-* Examples:
#D-
function IGCM_sys_Chmod {
  IGCM_debug_PushStack "IGCM_sys_Chmod" -- $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Chmod :" $@
  fi
  if [ $DRYRUN -le 1 ]; then
    \chmod $@
    if [ $? -gt 0 ] ; then
      echo "IGCM_sys_Chmod : erreur."
      IGCM_debug_Exit "IGCM_sys_Chmod"
    fi
  else
    ( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
  fi
  IGCM_debug_PopStack "IGCM_sys_Chmod"
}

#D-#==================================================
#D-function IGCM_sys_FileSize
#D-* Purpose: Filesize
#D-* Examples:
#D-
function IGCM_sys_FileSize {
  IGCM_debug_PushStack "IGCM_sys_FileSize" $@

  typeset sizeF
  set +A sizeF -- $( ls -la ${1} )
  if [ $? -gt 0 ] ; then
    IGCM_debug_Exit "IGCM_sys_FileSize"
  fi
  eval ${2}=${sizeF[4]}

  IGCM_debug_PopStack "IGCM_sys_FileSize"
}

#D-#==================================================
#D-function IGCM_sys_TestDir
#D-* Purpose: Test Directory that must exists
#D-* Examples:
#D-
function IGCM_sys_TestDir {
  IGCM_debug_PushStack "IGCM_sys_TestDir" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_TestDir :" $@
  fi
  typeset ExistFlag
  ExistFlag=$( [ -d $1 ] && echo 0 || echo 1 )
  IGCM_debug_PopStack "IGCM_sys_TestDir"

  return ${ExistFlag}
}

#D-#==================================================
#D-function IGCM_sys_TestDirArchive
#D-* Purpose: Test Directory that must exists on Archive
#D-* Examples:
#D-
function IGCM_sys_TestDirArchive {
  IGCM_debug_PushStack "IGCM_sys_TestDirArchive" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_TestDirArchive :" $@
  fi
  typeset ExistFlag
  ExistFlag=$( [ -d $1 ] && echo 0 || echo 1 )
  IGCM_debug_PopStack "IGCM_sys_TestDirArchive"

  return ${ExistFlag}
}

#D-#==================================================
#D-function IGCM_sys_IsFileArchived
#D-* Purpose: Test file that must NOT EXISTS on Archive
#D-* Examples:
#D-
function IGCM_sys_IsFileArchived {
  IGCM_debug_PushStack "IGCM_sys_IsFileArchived" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_IsFileArchived :" $@
  fi
  typeset IsArchivedFlag
  IsArchivedFlag=$( [ X$( echo $1 | grep ^\/dmnfs ) != X ] && echo 0 || echo 1 )
  IGCM_debug_PopStack "IGCM_sys_IsFileArchived"

  return ${IsArchivedFlag}
}

#D-#==================================================
#D-function IGCM_sys_TestFileArchive
#D-* Purpose: Test file that must NOT EXISTS on Archive
#D-* Examples:
#D-
function IGCM_sys_TestFileArchive {
  IGCM_debug_PushStack "IGCM_sys_TestFileArchive" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_TestFileArchive :" $@
  fi
  typeset ExistFlag
  ExistFlag=$( [ -f $1 ] && echo 0 || echo 1 )
  IGCM_debug_PopStack "IGCM_sys_TestFileArchive"

  return ${ExistFlag}
}

#D-#==================================================
#D-function IGCM_sys_TestFileBuffer
#D-* Purpose: Test file that must NOT EXISTS on Buffer
#D-* Examples:
#D-
function IGCM_sys_TestFileBuffer {
  IGCM_debug_PushStack "IGCM_sys_TestFileBuffer" $@
  typeset ExistFlag
  ExistFlag=$( [ -f $1 ] && echo 0 || echo 1 )
  IGCM_debug_PopStack "IGCM_sys_TestFileBuffer"

  return ${ExistFlag}
}

#D-#==================================================
#D-function IGCM_sys_CountFileArchive
#D-* Purpose: Count files on Archive filesystem
#D-* Examples:
#D-
function IGCM_sys_CountFileArchive {
  IGCM_debug_PushStack "IGCM_sys_CountFileArchive" $@
  ls ${@} 2>/dev/null | wc -l
  if [ $? -gt 0 ] ; then
    echo "IGCM_sys_CountFileArchive : erreur."
  fi
  IGCM_debug_PopStack "IGCM_sys_CountFileArchive"
}

#D-#==================================================
#D-function IGCM_sys_CountFileBuffer
#D-* Purpose: Count files on Scratch filesystem
#D-* Examples:
#D-
function IGCM_sys_CountFileBuffer {
  IGCM_debug_PushStack "IGCM_sys_CountFileBuffer" $@
  ls ${@} 2>/dev/null | wc -l
  if [ $? -gt 0 ] ; then
    echo "IGCM_sys_CountFileBuffer : erreur."
  fi
  IGCM_debug_PopStack "IGCM_sys_CountFileBuffer"
}

#D-#==================================================
#D-function IGCM_sys_Tree
#D-* Purpose: Tree directories with files on ${ARCHIVE}
#D-* Examples: IGCM_sys_Tree ${R_IN} ${R_OUT}
#D-
function IGCM_sys_Tree {
  IGCM_debug_PushStack "IGCM_sys_Tree" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Tree :" $@
  fi

  \ls -lR ${@}

  IGCM_debug_PopStack "IGCM_sys_Tree"
}

#D-#==================================================
#D-function IGCM_sys_Tar
#D-* Purpose: master tar command
#D-* Examples:
#D-
function IGCM_sys_Tar {
  IGCM_debug_PushStack "IGCM_sys_Tar" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Tar :" $@
  fi
  \tar cf $@
  if [ $? -gt 0 ] ; then
    echo "IGCM_sys_Tar : erreur."
    IGCM_debug_Exit "IGCM_sys_Tar"
  fi
  IGCM_debug_PopStack "IGCM_sys_Tar"
}

#D-#==================================================
#D-function IGCM_sys_UnTar
#D-* Purpose: master un-tar command
#D-* Examples:
#D-
function IGCM_sys_UnTar {
  IGCM_debug_PushStack "IGCM_sys_UnTar" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_UnTar :" $@
  fi
  \tar xvf $1
  if [ $? -gt 0 ] ; then
    echo "IGCM_sys_UnTar : erreur."
    IGCM_debug_Exit "IGCM_sys_UnTar"
  fi
  IGCM_debug_PopStack "IGCM_sys_UnTar"
}

#D-#==================================================
#D-function IGCM_sys_Qsub
#D-* Purpose: Qsub new job
#D-* Examples:
#D-
function IGCM_sys_Qsub {
  IGCM_debug_PushStack "IGCM_sys_Qsub" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Qsub :" $@
  fi
  /usr/bin/nqsII/qsub -o ${Script_Output} -N ${config_UserChoices_JobName}.${CumulPeriod} < $1
  if [ $? -gt 0 ] ; then
    echo "IGCM_sys_Qsub : erreur -o ${Script_Output} -N ${config_UserChoices_JobName}.${CumulPeriod} $@."
    IGCM_debug_Exit "IGCM_sys_Qsub"
  fi
  IGCM_debug_PopStack "IGCM_sys_Qsub"
}

#D-#==================================================
#D-function IGCM_sys_QsubPost
#D-* Purpose: Qsub new job on scalaire
#D-* Examples:
#D-
function IGCM_sys_QsubPost {
  IGCM_debug_PushStack "IGCM_sys_QsubPost" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_QsubPost :" $@
  fi

# ============ FRONTEND START ============ #

  /usr/bin/nqsII/qsub -q scalaire -o ${POST_DIR}/${Script_Post_Output}.out ${libIGCM}/$1.job -v ${listVarEnv}

# ============ FRONTEND  END  ============ #

# ============ CESIUM START ============ #
#  typeset NB_ESSAI DELAI status i
#  # number of tentative
#  NB_ESSAI=10
#  # time delay between tentative
#  DELAI=10
#  (( i = 0 ))
#  while [ $i -lt $NB_ESSAI ] ; do
#    /usr/local/bin/ccc_msub -o ${POST_DIR}/${Script_Post_Output}.out -e ${POST_DIR}/${Script_Post_Output}.e.out -E "-v ${listVarEnv}" ${libIGCM_POST}/$1.job
#    status=$?
#    if [ ${status} -gt 0 ]; then
#      sleep $DELAI
#    else
#      break
#    fi
#    (( i = i + 1 ))
#  done 
# ============ CESIUM  END  ============ #

  if [ $? -gt 0 ] ; then
    echo "IGCM_sys_QsubPost : erreur " $@
    IGCM_debug_Exit "IGCM_sys_QsubPost"
  fi
  IGCM_debug_PopStack "IGCM_sys_QsubPost"
}

#D-*************************
#D- File transfer functions 
#D-*************************
#D-

#D-#==================================================
#D-function IGCM_sys_Rsync_out
#D-* Purpose: treat return val of rsync
#D-* Examples: IGCM_sys_Rsync_out out_RET_rsync
#D-  Error values and explanations can depend on your system version.
function IGCM_sys_Rsync_out {
  RET=$1
  if [ ! $RET ] ; then
    echo "rsync error !"
  fi

  if [ $MYLANG = "fr" ]; then
    case $RET in
    0)  return ;;
    1)  echo "Erreur de rsync ; RERR_SYNTAX : "
      echo "Erreur de syntaxe ou d'utilisation."
      return;;
    2)  echo "Erreur de rsync ; RERR_PROTOCOL : "
      echo "Incompatibilit de protocole."
      return;;
    3)  echo "Erreur de rsync ; RERR_FILESELECT 3"
      echo "Erreurs  lors  de  la  slection des fichiers d'entre sortie et"
      echo "rpertoires"
      return;;
    4)  echo "Erreur de rsync ; RERR_UNSUPPORTED"
      echo "Action demande non supporte : une tentative de manipulation de"
      echo "fichiers  64-bits  sur une plate-forme qui ne les supporte pas a"
      echo "t faite ; ou une option qui est supporte par le  client  mais"
      echo "pas par le serveur a t spcifie."
      return;;
    10) echo "Erreur de rsync ; RERR_SOCKETIO"
      echo "Erreur dans le socket d'entre sortie"
      return;;
    11) echo "Erreur de rsync ; RERR_FILEIO"
      echo "Erreur d'entre sortie fichier"
      return;;
    12) echo "Erreur de rsync ; RERR_STREAMIO"
      echo "Erreur dans flux de donne du protocole rsync"
      return;;
    13) echo "Erreur de rsync ; RERR_MESSAGEIO"
      echo "Erreur avec les diagnostics du programme"
      return;;
    14) echo "Erreur de rsync ; RERR_IPC"
      echo "Erreur dans le code IPC"
      return;;
    20) echo "Erreur de rsync ; RERR_SIGNAL"
      echo "SIGUSR1 ou SIGINT reu"
      return;;
    21) echo "Erreur de rsync ; RERR_WAITCHILD"
      echo "Une erreur retourne par waitpid()"
      return;;
    22) echo "Erreur de rsync ; RERR_MALLOC"
      echo "Erreur lors de l'allocation des tampons de mmoire de coeur"
      return;;
    23) echo ""
      echo "Erreur fichier inexistant"
      return;;
    30) echo "Erreur de rsync ; RERR_TIMEOUT"
      echo "Temps d'attente coul dans l'envoi/rception de donnes"
      return;;
    *)  echo "Erreur de rsync : code de retour de rsync inconnu :" $RET
      return;;
    esac
  elif [ $MYLANG = "en" ] ; then
    case $RET in
    0)  return;;		
    1)  echo "rsync error : Syntax or usage error "
      return;;
    2)  echo "rsync error : Protocol incompatibility "
      return;;
    3)  echo "rsync error : Errors selecting input/output files, dirs"
      return;;
    4)  echo "rsync error : Requested action not supported: an attempt"
      echo "was made to manipulate 64-bit files on a platform that cannot support"
      echo "them; or an option was specified that is supported by the client and"
      echo "not by the server."
      return;;
    5)  echo "rsync error : Error starting client-server protocol"
      return;;
    10) echo "rsync error : Error in socket I/O "
      return;;
    11) echo "rsync error : Error in file I/O "
      return;;
    12) echo "rsync error : Error in rsync protocol data stream "
      return;;
    13) echo "rsync error : Errors with program diagnostics "
      return;;
    14) echo "rsync error : Error in IPC code "
      return;;
    20) echo "rsync error : Received SIGUSR1 or SIGINT "
      return;;
    21) echo "rsync error : Some error returned by waitpid() "
      return;;
    22) echo "rsync error : Error allocating core memory buffers "
      return;;
    23) echo "rsync error : Partial transfer due to error"
      return;;
    24) echo "rsync error : Partial transfer due to vanished source files"
      return;;
    30) echo "rsync error : Timeout in data send/receive "
      return;;
    *)  echo "rsync error : return code of rsync unknown :" $RET
      return;;
    esac
  else
    echo "unknown language $MYLANG."
    return
  fi
}

#D-#==================================================
#D-function IGCM_sys_Miror_libIGCM
#D-* Purpose: Mirror libIGCM PATH and lib to cesium
#D-* Examples:
#D-
function IGCM_sys_Mirror_libIGCM {
  IGCM_debug_PushStack "IGCM_sys_Mirror_libIGCM"
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Mirror_libIGCM"
  fi

  typeset RET DEST

  mkdir -p ${HOME}/MIRROR/${PATHlibIGCM}

  echo ${RSYNC} ${RSYNC_opt} ${libIGCM} ${HOME}/MIRROR/${PATHlibIGCM} > out_rsync 2>&1
  ${RSYNC} ${RSYNC_opt} ${libIGCM} ${HOME}/MIRROR/${PATHlibIGCM} >> out_rsync 2>&1
  RET=$?

  if [ ${RET} -gt 0 ] ; then
    echo "IGCM_sys_Mirror_libIGCM Warning : no libIGCM on cesium."
    cat out_rsync
  fi
  IGCM_debug_PopStack "IGCM_sys_Mirror_libIGCM"
}

#====================================================
#- Call IGCM_sys_Mirror_libIGCM now !
if ( $MirrorlibIGCM ) ; then
  IGCM_sys_Mirror_libIGCM
fi

#D-#==================================================
#D-function IGCM_sys_Cp
#D-* Purpose: generic cp
#D-* Examples:
#D-
function IGCM_sys_Cp {
  IGCM_debug_PushStack "IGCM_sys_Cp" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Cp :" $@
  fi

  typeset RET

  echo cp $@ > out_rsync 2>&1
  \cp $@ >> out_rsync 2>&1
  RET=$?

  if [ ${RET} -gt 0 ] ; then
    echo "IGCM_sys_Cp : error."
    cat out_rsync
    IGCM_debug_Exit "IGCM_sys_Cp"
  else
    \rm out_rsync
  fi
  IGCM_debug_PopStack "IGCM_sys_Cp"
}

#D-#==================================================
#D-function IGCM_sys_Rm
#D-* Purpose: generic rm
#D-* Examples:
#D-
function IGCM_sys_Rm {
  IGCM_debug_PushStack "IGCM_sys_Rm" -- $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Rm :" $@
  fi

  typeset RET

  echo rm $@ > out_rsync 2>&1
  \rm $@ >> out_rsync 2>&1
  RET=$?

  if [ ${RET} -gt 0 ] ; then
    echo "IGCM_sys_Rm : error."
    cat out_rsync
    IGCM_debug_Exit "IGCM_sys_Rm"
  else
    \rm out_rsync
  fi
  IGCM_debug_PopStack "IGCM_sys_Rm"
}

#D-#==================================================
#D-function IGCM_sys_Mv
#D-* Purpose: generic move
#D-* Examples:
#D-
function IGCM_sys_Mv {
  IGCM_debug_PushStack "IGCM_sys_Mv" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Mv :" $@
  fi

  if [ $DRYRUN = 0 ]; then

    typeset RET
    
    echo mv $@ > out_rsync 2>&1
    \mv $@ >> out_rsync 2>&1
    RET=$?
    
    if [ ${RET} -gt 0 ] ; then
      echo "IGCM_sys_Mv : error in mv."
      cat out_rsync
      IGCM_debug_Exit "IGCM_sys_Mv"
    else
      \rm out_rsync
    fi
  else
    ( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
  fi

  IGCM_debug_PopStack "IGCM_sys_Mv"
}

#D-#==================================================
#D-function IGCM_sys_Put_Dir
#D-* Purpose: Copy a complete directory on $(ARCHIVE)
#D-* Examples:
#D-
function IGCM_sys_Put_Dir {
  IGCM_debug_PushStack "IGCM_sys_Put_Dir" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Put_Dir :" $@
  fi
  if [ $DRYRUN = 0 ]; then
    if [ ! -d ${1} ] ; then
      echo "WARNING : IGCM_sys_Put_Dir ${1} DOES NOT EXIST ."
      IGCM_debug_PopStack "IGCM_sys_Put_Dir"
      return
    fi

    typeset RET

    # Only if we use rsync
    #IGCM_sys_TestDirArchive $( dirname $2 )
    #
    #USUAL WAY
    \cp -r $1 $2 > out_rsync 2>&1
    RET=$?

    if [ ${RET} -gt 0 ] ; then
      echo "IGCM_sys_Put_Dir : error."
      cat out_rsync
      IGCM_debug_Exit "IGCM_sys_Put_Dir"
    else
      \rm out_rsync
    fi
  else
    ( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
  fi
  IGCM_debug_PopStack "IGCM_sys_Put_Dir"
}

#D-#==================================================
#D-function IGCM_sys_Get_Dir
#D-* Purpose: Copy a complete directory from ${ARCHIVE}
#D-* Examples:
#D-
function IGCM_sys_Get_Dir {
  IGCM_debug_PushStack "IGCM_sys_Get_Dir" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Get_Dir :" $@
  fi
  if [ $DRYRUN = 0 ]; then
#    if [ ! -d ${1} ] ; then
#      echo "WARNING : IGCM_sys_Get_Dir ${1} DOES NOT EXIST ."
#      IGCM_debug_PopStack "IGCM_sys_Get_Dir"
#      return
#    fi

    typeset RET

    # Only if we use rsync
    #IGCM_sys_TestDirArchive $( dirname $2 )
    #
    # USUAL WAY
    # add dmfind/dmget (to demigrate all offline files) :
    #dmfind $1 -state MIG -o -state OFL -o -state PAR | dmget
    \cp -r $1 $2 > out_rsync 2>&1
    RET=$?

    if [ ${RET} -gt 0 ] ; then
      echo "IGCM_sys_Get_Dir : error."
      cat out_rsync
      IGCM_debug_Exit "IGCM_sys_Get_Dir"
    else
      \rm out_rsync
    fi
  else
    ( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
  fi
  IGCM_debug_PopStack "IGCM_sys_Get_Dir"
}

#D-#==================================================
#D-function IGCM_sys_Put_Rest
#D-* Purpose: Put computied restarts on ${ARCHIVE}.
#D-           File and target directory must exist.
#D-* Examples:
#D-
function IGCM_sys_Put_Rest {
  IGCM_debug_PushStack "IGCM_sys_Put_Rest" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Put_Rest :" $@
  fi
  if [ $DRYRUN = 0 ]; then
    if [ ! -f ${1} ] ; then
      echo "ERROR : IGCM_sys_Put_Rest ${1} DOES NOT EXIST ."
      IGCM_debug_Exit "IGCM_sys_Put_Rest"
    fi

    typeset RET
    #
    if [ X${JobType} = XRUN ] ; then
      IGCM_sys_Chmod 444 ${1}
    fi

    #
    # USUAL WAY
    \cp $1 $2 > out_rsync 2>&1
    RET=$?

# 	#RSYNC WITH NETWORK SSH CALL
# 	echo ${RSYNC} ${RSYNC_opt} -e ssh ${RUN_DIR}/$1 ${STOREHOST}:${2} > out_rsync 2>&1
# 	${RSYNC} ${RSYNC_opt} -e ssh ${RUN_DIR}/$1 ${STOREHOST}:${2} >> out_rsync 2>&1

#  	#RSYNC WITH NFS USE
#  	echo ${RSYNC} ${RSYNC_opt} ${RUN_DIR}/$1 ${2} > out_rsync 2>&1
#  	${RSYNC} ${RSYNC_opt} ${RUN_DIR}/$1 ${2} >> out_rsync 2>&1

#  	RET=$?
#  	IGCM_sys_Rsync_out $RET

#  	${libIGCM}/libIGCM_sys/IGCM_analyse_rsync_out.awk out_rsync
#  	(( RET=RET+$? ))

    if [ ${RET} -gt 0 ] ; then
      echo "IGCM_sys_Put_Rest : error."
      cat out_rsync
      IGCM_debug_Exit "IGCM_sys_Put_Rest"
    else
      \rm out_rsync
    fi
  else
    ( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
  fi
  IGCM_debug_PopStack "IGCM_sys_Put_Rest"
}

#D-#==================================================
#D-function IGCM_sys_PutBuffer_Rest
#D-* Purpose: Put computied restarts on ${SCRATCHDIR}.
#D-           File and target directory must exist.
#D-* Examples:
#D-
function IGCM_sys_PutBuffer_Rest {
  IGCM_debug_PushStack "IGCM_sys_PutBuffer_Rest" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_PutBuffer_Rest :" $@
  fi
  if [ $DRYRUN = 0 ]; then
    if [ ! -f ${1} ] ; then
      echo "ERROR : IGCM_sys_PutBuffer_Rest ${1} DOES NOT EXIST ."
      IGCM_debug_Exit "IGCM_sys_PutBuffer_Rest"
    fi

    typeset RET
    #
    if [ X${JobType} = XRUN ] ; then
      IGCM_sys_Chmod 444 ${1}
    fi

    #
    # USUAL WAY
    \cp $1 $2 > out_rsync 2>&1
    RET=$?

    if [ ${RET} -gt 0 ] ; then
      echo "IGCM_sys_PutBuffer_Rest : error."
      cat out_rsync
      IGCM_debug_Exit "IGCM_sys_PutBuffer_Rest"
    else
      \rm out_rsync
    fi
  else
    ( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
  fi
  IGCM_debug_PopStack "IGCM_sys_PutBuffer_Rest"
}

#D-#==================================================
#D-function IGCM_sys_Put_Out
#D-* Purpose: Copy a file on ${ARCHIVE} after having chmod it in readonly
#D-* Examples:
#D-
function IGCM_sys_Put_Out {
  IGCM_debug_PushStack "IGCM_sys_Put_Out" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Put_Out :" $@
  fi
  if [ $DRYRUN = 0 ]; then
    if [ ! -f ${1} ] ; then
      echo "WARNING : IGCM_sys_Put_Out ${1} DOES NOT EXIST ."
      IGCM_debug_PopStack "IGCM_sys_Put_Out"
      return 1
    fi
    #
    IGCM_sys_MkdirArchive $( dirname $2 )
    #
    typeset RET

    #=====================================================
    #         COMMENT OUT DOUBLE COPY ON SCRATCHDIR
    #=====================================================

    #echo ${2} | grep "${R_OUT}" > /dev/null 2>&1
    #if [ $? -eq 0 ] ; then
    #    typeset WORKPATH FILEPATH
    #    WORKPATH=$( dirname $2 | sed -e "s|${R_OUT}|${R_BUF}|" )
    #    IGCM_sys_MkdirWork ${WORKPATH} 
    #    FILEPATH=${WORKPATH}/$( basename $2 )
    #    #
    #    IGCM_sys_Cp ${1} ${FILEPATH}
    #fi

    if [ X${JobType} = XRUN ] ; then
      if [ X${3} = X ] ; then
	IGCM_sys_Chmod 444 ${1}
      fi
    fi
    #
    # USUAL WAY
    \cp $1 $2 > out_rsync 2>&1
    RET=$?

# 	#RSYNC WITH NETWORK SSH CALL
# 	echo ${RSYNC} ${RSYNC_opt} -e ssh ${RUN_DIR}/$1 ${STOREHOST}:${2} > out_rsync 2>&1
# 	${RSYNC} ${RSYNC_opt} -e ssh ${RUN_DIR}/$1 ${STOREHOST}:${2} >> out_rsync 2>&1

#  	#RSYNC WITH NFS USE
#  	echo ${RSYNC} ${RSYNC_opt} ${RUN_DIR}/$1 ${2} > out_rsync 2>&1
#  	${RSYNC} ${RSYNC_opt} ${RUN_DIR}/$1 ${2} >> out_rsync 2>&1

#  	RET=$?
#  	IGCM_sys_Rsync_out $RET

#  	${libIGCM}/libIGCM_sys/IGCM_analyse_rsync_out.awk out_rsync
#  	(( RET=RET+$? ))

    if [ ${RET} -gt 0 ] ; then
      echo "IGCM_sys_Put_Out : error."
      cat out_rsync
      IGCM_debug_Exit "IGCM_sys_Put_Out"
    else
      \rm out_rsync
    fi
  else
    ( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
  fi
  IGCM_debug_PopStack "IGCM_sys_Put_Out"
  return 0
}

#D-#==================================================
#D-function IGCM_sys_PutBuffer_Out
#D-* Purpose: Copy a file on ${SCRATCHDIR} after having chmod it in readonly
#D-* Examples:
#D-
function IGCM_sys_PutBuffer_Out {
  IGCM_debug_PushStack "IGCM_sys_PutBuffer_Out" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_PutBuffer_Out :" $@
  fi
  if [ $DRYRUN = 0 ]; then
    if [ ! -f ${1} ] ; then
      echo "WARNING : IGCM_sys_PutBuffer_Out ${1} DOES NOT EXIST ."
      IGCM_debug_PopStack "IGCM_sys_PutBuffer_Out"
      return 1
    fi
    #
    IGCM_sys_Mkdir $( dirname $2 )
    #
    typeset RET

    if [ X${JobType} = XRUN ] ; then
      if [ X${3} = X ] ; then
	IGCM_sys_Chmod 444 ${1}
      fi
    fi
    #
    # USUAL WAY
    \cp $1 $2 > out_rsync 2>&1
    RET=$?

    if [ ${RET} -gt 0 ] ; then
      echo "IGCM_sys_PutBuffer_Out : error."
      cat out_rsync
      IGCM_debug_Exit "IGCM_sys_PutBuffer_Out"
    else
      \rm out_rsync
    fi
  else
    ( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
  fi
  IGCM_debug_PopStack "IGCM_sys_PutBuffer_Out"
  return 0
}

#D-#==================================================
#D-function IGCM_sys_Get
#D-* Purpose: Get a file from ${ARCHIVE}
#D-* Examples: IGCM_sys_Get myfile /destpath/myfile_with_PREFIX
#D-            IGCM_sys_Get /l Array_contain_myfiles /destpath/
function IGCM_sys_Get {
  IGCM_debug_PushStack "IGCM_sys_Get" $@

  typeset DEST RET dm_liste ifile target file_work 

  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_Get :" $@
  fi
  if [ $DRYRUN -le 2 ]; then
    if [ X${1} = X'/l' ] ; then
      # test if the first file is present in the old computation :
      eval set +A dm_liste \${${2}}
    else
      eval set +A dm_liste ${1}
    fi
    eval DEST=\${${#}}

    #=====================================================
    #         COMMENT OUT DOUBLE COPY ON SCRATCHDIR
    #=====================================================

    # Is it an R_OUT file (not R_IN) ?
    #echo ${dm_liste[0]} | grep "${R_OUT}" > /dev/null 2>&1
    #if [ $? -eq 0 ] ; then
    #    # Yes  ? then we try to get it in SCRATCHDIR
    #    set +A file_work $( echo ${dm_liste[*]} | sed -e "s|${R_OUT}|${R_BUF}|g" )
    #    if [ -f ${file_work[0]} ] ; then
    #	IGCM_sys_Cp ${file_work[*]} ${DEST}
    #	IGCM_debug_PopStack "IGCM_sys_Get"
    #	return
    #    fi
    #fi

    # test if the (first) file is present in the old computation :
    IGCM_sys_TestFileArchive ${dm_liste[0]}
    RET=$?
    if [ ${RET} -gt 0 ] ; then
      echo "IGCM_sys_Get, ERROR : regular file ${dm_liste[0]} DOES NOT EXIST ."
      IGCM_debug_Exit "IGCM_sys_Get"
    fi

    #dmget ${dm_liste[*]} > out_rsync 2>&1
    ccc_hsm get ${dm_liste[*]} > out_rsync 2>&1
    RET=$?
    if [ ${RET} -gt 0 ] ; then
      echo "WARNING IGCM_sys_Get : demigration error."
      cat out_rsync
      echo "WARNING IGCM_sys_Get : will stop later if the cp fails."
    fi

    #if [ ${RET} -gt 0 ] ; then
    #    if [ ! "X$( grep "Lost dmusrcmd connection" out_rsync )" = "X" ] ; then
    #	cat out_rsync
    #	echo "WARNING IGCM_sys_Get : Lost dmusrcmd connection : "
    #	sleep 30
    #	echo "We try another time"
    ##	dmget ${dm_liste[*]} > out_rsync 2>&1
    #   ccc_hsm get ${dm_liste[*]} > out_rsync 2>&1
    #	RET=$?
    #	if [ ${RET} -gt 0 ] ; then
    #	    echo "ERROR IGCM_sys_Get : again demigration error :"
    #	    cat out_rsync
    #	    IGCM_debug_Exit "IGCM_sys_Get"
    #	fi
    #    else
    #	echo "ERROR IGCM_sys_Get : demigration error :"
    #	cat out_rsync
    #	IGCM_debug_Exit "IGCM_sys_Get"
    #    fi
    #fi

    #USUAL WAY
    if [ X${1} = X'/l' ] ; then
      (( RET=0 ))
      for target in ${dm_liste[*]} ; do
	local_file=$( basename ${target} )
	\cp ${target} ${DEST}/${local_file} >> out_rsync 2>&1
	(( RET = RET + $? ))
      done
    else
      \cp ${dm_liste} ${DEST} >> out_rsync 2>&1
      RET=$?
    fi

# 	#RSYNC WITH NETWORK SSH CALL
# 	echo ${RSYNC} ${RSYNC_opt} -e ssh ${STOREHOST}:"${dm_liste}" ${STOREHOST}:${RUN_DIR}/${DEST} > out_rsync 2>&1
# 	${RSYNC} ${RSYNC_opt} -e ssh ${STOREHOST}:"${dm_liste}" ${STOREHOST}:${RUN_DIR}/${DEST} >> out_rsync 2>&1

#  	#RSYNC WITH NFS USE
#  	echo ${RSYNC} ${RSYNC_opt} ${dm_liste} ${RUN_DIR}/${DEST} > out_rsync 2>&1
#  	${RSYNC} ${RSYNC_opt} ${dm_liste} ${RUN_DIR}/${DEST} >> out_rsync 2>&1

# 	RET=$?
#  	IGCM_sys_Rsync_out $RET

#  	${libIGCM}/libIGCM_sys/IGCM_analyse_rsync_out.awk out_rsync
#  	(( RET=RET+$? ))

    if [ ${RET} -gt 0 ] ; then
      echo "IGCM_sys_Get : copy error."
      cat out_rsync
      IGCM_debug_Exit "IGCM_sys_Get"
    else
      \rm out_rsync
    fi
  else
    ( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
  fi
  IGCM_debug_PopStack "IGCM_sys_Get"
}

#D-#==================================================
#D-function IGCM_sys_GetBuffer
#D-* Purpose: Get a file from ${SCRATCHDIR}
#D-* Examples: IGCM_sys_GetBuffer myfile /destpath/myfile_with_PREFIX
#D-            IGCM_sys_GetBuffer /l Array_contain_myfiles /destpath/
function IGCM_sys_GetBuffer {
  IGCM_debug_PushStack "IGCM_sys_GetBuffer" $@

  typeset DEST RET buf_liste ifile target file_work 

  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_GetBuffer :" $@
  fi
  if [ $DRYRUN -le 2 ]; then
    if [ X${1} = X'/l' ] ; then
      # test if the first file is present in the old computation :
      eval set +A buf_liste \${${2}}
    else
      eval set +A buf_liste ${1}
    fi
    eval DEST=\${${#}}

    #USUAL WAY
    if [ X${1} = X'/l' ] ; then
      (( RET=0 ))
      for target in ${buf_liste[*]} ; do
	local_file=$( basename ${target} )
	\cp ${target} ${DEST}/${local_file} >> out_rsync 2>&1
	(( RET = RET + $? ))
      done
    else
      \cp ${buf_liste} ${DEST} >> out_rsync 2>&1
      RET=$?
    fi

    if [ ${RET} -gt 0 ] ; then
      echo "IGCM_sys_GetBuffer : copy error."
      cat out_rsync
      IGCM_debug_Exit "IGCM_sys_GetBuffer"
    else
      \rm out_rsync
    fi
  else
    ( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
  fi
  IGCM_debug_PopStack "IGCM_sys_GetBuffer"
}

#D-#==================================================
#D-function IGCM_sys_GetDate_FichWork
#D-* Purpose: donne la date filesys d'un fichier sur le filesystem WORK
#D-* Examples:
#D-
function IGCM_sys_GetDate_FichWork {
  IGCM_debug_PushStack "IGCM_sys_FichWork" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_GetDate_FichWork :" $@
  fi
  typeset dateF
  set +A dateF -- $( ls -l --full-time --time-style='+%Y%m%d%H%M%S' ${1} )
  eval ${2}=${dateF[5]}

    # donne la date filesys d'un fichier sur la machine work
  IGCM_debug_PopStack "IGCM_sys_FichWork"
}

#D-#==================================================
#D-function IGCM_sys_GetDate_FichArchive
#D-* Purpose: donne la date filesys d'un fichier sur le filesystem ARCHIVE 
#D-* Examples:
#D-
function IGCM_sys_GetDate_FichArchive {
  IGCM_debug_PushStack "IGCM_sys_FichArchive" $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_GetDate_FichArchive :" $@
  fi
  typeset dateF
  set +A dateF -- $( ls -l --full-time --time-style='+%Y%m%d%H%M%S' ${1} )
  eval ${2}=${dateF[5]}

  IGCM_debug_PopStack "IGCM_sys_FichArchive"
}

##############################################################
# REBUILD OPERATOR

function IGCM_sys_rebuild {
  IGCM_debug_PushStack "IGCM_sys_rebuild" -- $@
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_rebuild :" $@
  fi
  /home/cont003/p86ipsl/SX8/bin/rebuild -f -o $@
  if [ $? -gt 0 ] ; then
    echo "IGCM_sys_rebuild : erreur ${@}."
    IGCM_debug_Exit "rebuild"
  fi

  IGCM_debug_PopStack "IGCM_sys_rebuild"
}

############################################################
# Activate Running Environnment Variables

function IGCM_sys_activ_variables {
  IGCM_debug_PushStack "IGCM_sys_activ_variables"
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_activ_variables"
  fi

# --------------------------------------------------------------------
#D- MPI specifications
# --------------------------------------------------------------------

#D-- MPISUSPEND
  export MPISUSPEND=${MPISUSPEND:=OFF}

#D-- MPIPROGINF #other choices : ALL_DETAIL2
  export MPIPROGINF=ALL
#D- activate ftrace (with -ftrace)
  export F_FTRACE=YES
#D- communication information (with -ftrace)
  export MPICOMMINF=DETAIL

# --------------------------------------------------------------------
#D- Other specifications
# --------------------------------------------------------------------

#D- max number of character/line in output job
  export F_SYSLEN=5000
#D- number of error that can be admitted on the NEC
  export F_ERRCNT=0
#D- global performance
  export F_PROGINF=DETAIL

#D- I/O performance (FORTRAN I/O only not netCDF)
  export F_FILEINF=${F_FILEINF:=NO}
#D- netCDF I/O performance
  export NC_FILEINF=${NC_FILEINF:=NO}

  IGCM_debug_PopStack "IGCM_sys_activ_variables"
}

############################################################
# Desactivate Running Environnment Variables

function IGCM_sys_desactiv_variables {
  IGCM_debug_PushStack "IGCM_sys_desactiv_variables"
  if ( $DEBUG_sys ) ; then
    echo "IGCM_sys_desactiv_variables"
  fi
# --------------------------------------------------------------------
#D- MPI specifications
# --------------------------------------------------------------------

#D-- MPIPROGINF
  export MPIPROGINF=NO

# --------------------------------------------------------------------
#D- Other specifications
# --------------------------------------------------------------------

#D- global performance
  export F_PROGINF=NO  

  IGCM_debug_PopStack "IGCM_sys_desactiv_variables"
}

############################################################
# Build MPI/OMP scripts
function IGCM_sys_build_execution_scripts 
{
    IGCM_debug_PushStack "IGCM_sys_build_execution_scripts" $@
    if ( $DEBUG_sys ) ; then
        echo "IGCM_sys_build_execution_scripts " $@
    fi

    typeset NbNodes_Job NbProc_Job comp_proc_mpi_loc comp_proc_omp_loc mpi_count
    
    if [ ! -f ${SUBMIT_DIR}/Job_${config_UserChoices_JobName} ]  ; then
        IGCM_debug_Exit "IGCM_sys_mercurex9 build_execution_scripts : Job_${config_UserChoices_JobName} don't exist in SUBMIT_DIR : ${SUBMIT_DIR} "
    fi

    if ( ${OK_PARA_MPMD} ) ; then

        if [ -f run_file ] ; then
            IGCM_sys_Rm -f run_file
        fi
        touch run_file

	if ( ${OK_PARA_OMP} ) ; then
		IGCM_debug_Print 2 "Error in config.card on SX9 : OpenMP not available"
		exit 1
	else
# OLD :
#PBS -v BATCH_NUM_PROC_TOT=4
#PBS -l cpunum_job=${BATCH_NUM_PROC_TOT}
	    echo "Job_${config_UserChoices_JobName} includes BATCH_NUM_PROC_TOT = \c"
	    cat ${SUBMIT_DIR}/Job_${config_UserChoices_JobName} | grep -- "-v *BATCH_NUM_PROC_TOT" | sed -e "s/.*BATCH_NUM_PROC_TOT *= *//" 
	    NbProc_Job=$( cat ${SUBMIT_DIR}/Job_${config_UserChoices_JobName} | grep -- "-v *BATCH_NUM_PROC_TOT" | sed -e "s/.*BATCH_NUM_PROC_TOT *= *//" )
	    NbProc_Job=${NbProc_Job:=0}
	    if [ ${NbProc_Job} -eq 0 ] ; then
		IGCM_debug_Print 2 "Error in Job_${config_UserChoices_JobName} ressources : no BATCH_NUM_PROC_TOT defined with MPI only run."
		exit 1
	    fi
	fi

# run_file construction

# Then first loop on the components for the coupler ie oasis

### the coupler ie oasis must be the first one
    for comp in ${config_ListOfComponents[*]} ; do
	
	eval ExeNameIn=\${config_Executable_${comp}[0]}
	eval ExeNameOut=\${config_Executable_${comp}[1]}
	    
	# for CPL component only
	if [ "X${comp}" = "XCPL" ] ; then

	    eval comp_proc_mpi_loc=\${${comp}_PROC_MPI}
	    eval comp_proc_omp_loc=\${${comp}_PROC_OMP}
	    
	    if ( ${OK_PARA_MPI} ) ; then 
	        echo "-p ${comp_proc_mpi_loc} -e ./${ExeNameOut}" >> run_file 
	    fi
	fi
    done

# Then second loop on the components

    for comp in ${config_ListOfComponents[*]} ; do
	
	eval ExeNameIn=\${config_Executable_${comp}[0]}
	eval ExeNameOut=\${config_Executable_${comp}[1]}
	    
	# Only if we really have an executable for the component and not the coupler ie oasis:
	if ( [ "X${ExeNameOut}" != X\"\" ] && [ "X${comp}" != "XCPL" ] ) ; then

	    eval comp_proc_mpi_loc=\${${comp}_PROC_MPI}
	    eval comp_proc_omp_loc=\${${comp}_PROC_OMP}
	    
	    if ( ${OK_PARA_MPI} ) ; then 
		    echo "-p ${comp_proc_mpi_loc} -e ./${ExeNameOut}" >> run_file 
	    fi
	fi
    done

    EXECUTION="${HOST_MPIRUN_COMMAND} -f ./run_file"

    IGCM_sys_Chmod u+x run_file
    if ( $DEBUG_sys ) ; then
      echo "run_file contains : "
      cat run_file
    fi

else # Only one executable. launch it.

    for comp in ${config_ListOfComponents[*]} ; do

        eval ExeNameIn=\${config_Executable_${comp}[0]}
        eval ExeNameOut=\${config_Executable_${comp}[1]}

	[ "X${ExeNameOut}" != X\"\" ] &&  EXECUTION="time mpirun -np ${BATCH_NUM_PROC_TOT} ./${ExeNameOut}"

    done

fi

    IGCM_debug_Print 1 "sys mercurex9 : execution command is "
    IGCM_debug_Print 1 "$EXECUTION"

    IGCM_debug_PopStack "IGCM_sys_build_execution_scripts"

}

function IGCM_sys_build_run_file {

IGCM_debug_Print 3 " dummy function : IGCM_sys_build_run_file "

}

